
#    MIDAS Httpd Server to provide Data Acquisition Control Services

package require MIDASDomainHandler       ;#  load default support procedures

global MIDAS_DataAcquisitionControlServer_Exists
set MIDAS_DataAcquisitionControlServer_Exists 1

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global MIDAS_DataAcquisitionControlServer_Implementation
set MIDAS_DataAcquisitionControlServer_Implementation 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
set MIDAS_DataAcquisitionCode 2     ;# initially reset

global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo
set MIDAS_DataAcquisitionError 0
set MIDAS_DataAcquisitionInfo "no error present"


#    state returns

#    code (int)  state (string)  reason  (string)

#       0           stopped
#       1           going
#       2           reset    "please setup"
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error           ******
#       7           changing to reset
#       8           undefined       ******
#       9           executing

set MIDAS_DataAcquisitionState(0) "stopped"
set MIDAS_DataAcquisitionState(1) "going"
set MIDAS_DataAcquisitionState(2) "reset"
set MIDAS_DataAcquisitionState(3) "setting up"
set MIDAS_DataAcquisitionState(4) "changing to going"
set MIDAS_DataAcquisitionState(5) "changing to stopped"
set MIDAS_DataAcquisitionState(6) "error"
set MIDAS_DataAcquisitionState(7) "changing to reset"
set MIDAS_DataAcquisitionState(8) "undefined"
set MIDAS_DataAcquisitionState(9) "executing"

set MIDAS_DataAcquisitionReason(0) ""
set MIDAS_DataAcquisitionReason(1) ""
set MIDAS_DataAcquisitionReason(2) "please setup"
set MIDAS_DataAcquisitionReason(3) "please wait"
set MIDAS_DataAcquisitionReason(4) "please wait"
set MIDAS_DataAcquisitionReason(5) "please wait"
set MIDAS_DataAcquisitionReason(6) "try reset"
set MIDAS_DataAcquisitionReason(7) "please wait"
set MIDAS_DataAcquisitionReason(8) "try reset"
set MIDAS_DataAcquisitionReason(9) "please wait"

global MIDAS_DataAcquisitionCounters MIDAS_DataAcquisitionRates

#    the order of counters and rates returned is

#    1  good events   (counter 0)
#    2  bad events    (counter 11)
#    3  input data buffers  (for non buffered input return 0)
#    4  output data buffers  (counter 4)

set MIDAS_DataAcquisitionCounters(0) 0
set MIDAS_DataAcquisitionCounters(1) 0
set MIDAS_DataAcquisitionCounters(2) 0
set MIDAS_DataAcquisitionCounters(3) 0

set MIDAS_DataAcquisitionRates(0) 0
set MIDAS_DataAcquisitionRates(1) 0
set MIDAS_DataAcquisitionRates(2) 0
set MIDAS_DataAcquisitionRates(3) 0


# Use namespaces to isolate our methods
namespace eval urn:DataAcquisitionControlServer {

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc ServerImplementation {opt} {
        do_ServerImplementation $opt
        return [rpcvar::rpcvar int 0]
    }

    proc GetLastError {args} {
        set vl [do_GetLastError]
        return [list [list ResponseCode int [lindex $vl 0]] [list Code int [lindex $vl 1]]  [list Reason string [lindex $vl 2]]]
    }

    proc GetInformation {args} {
        set vl [do_GetInformation]
        return [list [list ResponseCode int [lindex $vl 0]] [list Reason string [lindex $vl 1]]]
    }

    proc GetState {args} {
        set vl [do_GetState]
        return [list [list ResponseCode int [lindex $vl 0]] [list Code int [lindex $vl 1]]  [list State string [lindex $vl 2]]  [list Reason string [lindex $vl 3]]]
    }

    proc DoReset {args} {
        set vl [do_DoReset $args]
        return [rpcvar::rpcvar int $vl]
    }

    proc DoSetUp {args} {
        set vl [do_DoSetUp $args]
        return [rpcvar::rpcvar int $vl]
    }

    proc DoStop {args} {
        set vl [do_DoStop $args]
        return [rpcvar::rpcvar int $vl]
    }

    proc DoGo {args} {
        set vl [do_DoGo $args]
        return [rpcvar::rpcvar int $vl]
    }

    proc DoOption {args} {
        set vl [do_DoOption $args]
        return [rpcvar::rpcvar int $vl]
    }

    proc GetOption {args} {
        set vl [do_GetOption $args]
        return [list [list ResponseCode int 0] [list Option string $vl]]
    }

    proc GetCounters {args} {
        set vl [do_GetCounters]
        return [list [list ResponseCode int 0] [list Counters string $vl]]
    }

    proc GetRates {args} {
        set vl [do_GetRates]
        return [list [list ResponseCode int 0] [list Rates string $vl]]
    }

    proc do_ServerVerbose {option} {
        global MIDAS_DataAcquisitionControlServer_Verbose
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method ServerVerbose with parameter $option"}

        set MIDAS_DataAcquisitionControlServer_Verbose $option

        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "ServerVerbose now set to $MIDAS_DataAcquisitionControlServer_Verbose"}
    }

    proc do_ServerImplementation {option} {
        global MIDAS_DataAcquisitionControlServer_Verbose  MIDAS_DataAcquisitionControlServer_Implementation
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method ServerImplementation with parameter $option"}
        set MIDAS_DataAcquisitionControlServer_Implementation $option
    }

    proc do_GetState {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

       do_Implementation-GetState

       return [list 0 $MIDAS_DataAcquisitionCode [set MIDAS_DataAcquisitionState($MIDAS_DataAcquisitionCode)] [set MIDAS_DataAcquisitionReason($MIDAS_DataAcquisitionCode)]]
    }

    proc do_GetLastError {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

       return [list 0 $MIDAS_DataAcquisitionError $MIDAS_DataAcquisitionInfo]
    }

    proc do_GetInformation {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

       return [list 0 "No further informaton available at present"]
    }

    proc do_DoSetUp {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo
        global MIDAS_DataAcquisitionControlServer_Verbose
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method do_DoSetUp with parameter $args ([llength $args])"}

       if {$MIDAS_DataAcquisitionCode == 2} {
           set MIDAS_DataAcquisitionCode 3                 ;#  state change to setting up
           do_Implementation-SetUp  $args                       ;#  application specific
           set rc 0
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               set rc 1
               set MIDAS_DataAcquisitionError 4097
               set MIDAS_DataAcquisitionInfo "currently in wrong state for requested action"
           } else {
               return -code error -errorcode 4097 -errorinfo "currently in wrong state for requested action"
           }
       }
       if {$MIDAS_DataAcquisitionControlServer_Verbose} \
             {puts "exit method do_DoSetUp with rc=$rc & MIDAS_DataAcquisitionCode=$MIDAS_DataAcquisitionCode"}
       return $rc
    }

    proc do_DoReset {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

           set MIDAS_DataAcquisitionCode 7                ;#  state change to resetting 
           do_Implementation-Reset  $args                      ;#  application specific
           set rc 0

       return $rc
    }

    proc do_DoStop {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

       if {$MIDAS_DataAcquisitionCode == 1} {
           set MIDAS_DataAcquisitionCode 5               ;#  state change to stopping
           do_Implementation-Stop $args                       ;#  application specific
           set rc 0
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               set rc 1
               set MIDAS_DataAcquisitionError 4097
               set MIDAS_DataAcquisitionInfo "currently in wrong state for requested action"
           } else {
               return -code error -errorcode 4097 -errorinfo "currently in wrong state for requested action"
           }
       }
       return $rc
    }

    proc do_DoGo {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

       if {$MIDAS_DataAcquisitionCode == 0} {
           set MIDAS_DataAcquisitionCode 4                 ;#  state change to going
           do_Implementation-Go  $args                          ;#  application specific
           set rc 0
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               set rc 1
               set MIDAS_DataAcquisitionError 4097
               set MIDAS_DataAcquisitionInfo "currently in wrong state for requested action"
           } else {
               return -code error -errorcode 4097 -errorinfo "currently in wrong state for requested action"
           }
       }
       return $rc
    }

    proc do_DoOption {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo
        global MIDAS_DataAcquisitionControlServer_Verbose
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method do_DoOption with parameter $args ([llength $args])"}

# The args are in the form <option name> <value>
# It's up to the application to decide if it is (or isn't) in the wrong state for setting the option

       if {[llength $args] == 1} {set args [first $args]}   ;# SOAP ?
       if {[llength $args] == 1} {set args [first $args]}   ;# SOAP ?

       if {[llength $args] == 2} {
           set SAV $MIDAS_DataAcquisitionCode              ;#  existing state
           set MIDAS_DataAcquisitionCode 9                 ;#  state change to executing
           do_Implementation-SetOption  $SAV [first $args] [second $args]  
           set rc 0
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               set rc 1
               set MIDAS_DataAcquisitionError 4098
               set MIDAS_DataAcquisitionInfo "invalid format for option"
           } else {
               return -code error -errorcode 4097 -errorinfo "invalid format for option"
           }
       }
       if {$MIDAS_DataAcquisitionControlServer_Verbose} \
             {puts "exit method do_DoOption with rc=$rc & MIDAS_DataAcquisitionCode=$MIDAS_DataAcquisitionCode"}
       return $rc
    }

    proc do_GetOption {args} {
       global env MIDAS_DataAcquisitionControlServer_Implementation
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

# The arg is in the form <option name>
# It's up to the application to decide if it is (or isn't) in the wrong state for getting the option

       if {[llength $args] == 1} {
           set SAV $MIDAS_DataAcquisitionCode              ;#  existing state
           set MIDAS_DataAcquisitionCode 9                 ;#  state change to executing
           set rc [do_Implementation-GetOption $SAV [first $args]]
           set MIDAS_DataAcquisitionCode $SAV              ;#  restore state     
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               set rc 1
               set MIDAS_DataAcquisitionError 4098
               set MIDAS_DataAcquisitionInfo "invalid format for option"
           } else {
               return -code error -errorcode 4097 -errorinfo "invalid format for option"
           }
       }
       return $rc
    }

    proc do_GetCounters {} {
       global MIDAS_DataAcquisitionCounters

       set rc [do_Implementation-Counters]                            ;#  application specific
       return $rc
    }

    proc do_GetRates {} {
       global MIDAS_DataAcquisitionRates

       set rc [do_Implementation-Rates]                            ;#  application specific
       return $rc
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export ServerImplementation
     SOAP::export GetState
     SOAP::export DoSetUp
     SOAP::export DoReset
     SOAP::export DoStop
     SOAP::export DoGo
     SOAP::export DoOption
     SOAP::export GetOption
     SOAP::export GetCounters
     SOAP::export GetRates
     SOAP::export GetLastError
     SOAP::export GetInformation
}

# register this service with tclhttpd
set rc [SOAP::Domain::register  -prefix  /DataAcquisitionControlServer -namespace urn:DataAcquisitionControlServer -uri urn:DataAcquisitionControlServer]

proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DataAcquisitionControlServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

package provide DataAcquisitionControlServer 1.0

puts $rc


set msg "$rc Web Service Installed\n"

   if {[info procs ::DefineMessage] != "::DefineMessage"} {puts "DefineMessage unknown"} else {
       ::DefineMessage -c daqcontrol -l INFORMATION  "$msg"}



